% The following code is a denoising demo using Laplacian pyramid 
% filterbanks
% 
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

clear all; close all; clc; 

global GLOBAL_CSFB;
CSFB_start;

%% =====================================================================
%% ============================= choose data ===========================
%% =====================================================================

%% ------------------- Minnesota traffic graph -------------------------
% A is the structure on which the graph signal f is defined.
im=0;
load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_traffic_graph.mat']);
A = double(A>0);
load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_graph_signal.mat']);
locationsA=xy;
% remove two nodes (disconnected):
A(348:349,:)=[];A(:,348:349)=[];
locationsA(348:349,:)=[];
f(348:349)=[];
N=length(f); A=sparse(A);

% if you want to analyse a "smooth" signal instead of the default
% piece-wise constant signal:
L=CSFB_create_laplacian(A,'combinatorial');
[U,D]=eig(full(L)); U=U(:,1:5);
if U(1,1)<0
U=-U;end; f=sum(U,2); f=f./max(abs(f));

%% =====================================================================
%% ========================= choose parameters =========================
%% =====================================================================

G.A=A; G.N=length(f);
G.lap_type='combinatorial';
G.L=CSFB_create_laplacian(A,'combinatorial');
fulleigen_flag=1;
sparsify_flag=1;

% choose the level of corruption of the graph signal
sigm=0.25; %std of additive Gaussian noise


%% corrupt the graph signal with additive Gaussian noise
f_noisy=f+sigm.*randn(N,1);

%% =====================================================================
%% ================ decompose, threshold and recover ===================
%% =====================================================================
fprintf('Decompose, threshold and recover with the Laplacian pyramid...');
thresh=3*sigm;
frec=denoise_lap_pyramid(G,f_noisy,thresh,fulleigen_flag,sparsify_flag);
fprintf('\t\t\t\t Done.\n\n');
  
%% =====================================================================
%% =========================== plot results ============================
%% =====================================================================

fprintf('Plotting results...');
CLim=[min([f_noisy;f;frec]),max([f_noisy;f;frec])];

f1=figure; a1 = axes('parent',f1); CSFB_ggplot(A,locationsA,10,0,0,f,CLim);box off; axis off;
title(a1,'f original');
set(f1,'position',[90   529   560   420]);

f2=figure; a2 = axes('parent',f2);CSFB_ggplot(A,locationsA,10,0,0,f_noisy,CLim);box off; axis off;
title(a2,['noisy f (MSE=' num2str(mse(f,f_noisy),3) ', SNR=' num2str(snr(f,f_noisy-f),3) ')']);
set(f2,'position',[652   557   560   420]);

f3=figure; a3 = axes('parent',f3);CSFB_ggplot(A,locationsA,10,0,0,frec,CLim);box off; axis off;
title(a3,['f denoised (MSE=' num2str(mse(f,frec),3) ', SNR=' num2str(snr(f,frec-f),3) ')']);
set(f3,'position',[1218  557   560  420]);
fprintf('\t\t\t\t\t\t\t\t\t Done.\n\n');

