% The following code runs the Louvain clustering algorithm on A.
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function COM=CSFB_cluster_Louvain(A)

mkdir('temp');

%% translate structure information in bin format readable by the Louvain algorithm: 
% get link list:
[I,J,val]=find(A); 
cd('temp');
% create bin file thanks to the following mex function:
mex_convert_to_bin(I-1,J-1,val);
cd ..;

%% run Louvain
% we add here a dirty hack due to the fact that Louvain may output
% communities that are in fact disconnected...
% Ideally, this cell should only read: 
% COM=uint32(CSFB_run_Louvain(A));
% Instead, we run Louvain until it finds a partition in connected
% communities.. See more comments in the README file
disconnected_subgraph = 1;
MAX_ITER = 100;
ITER = 0;
while disconnected_subgraph == true
    ITER = ITER + 1;
    if ITER > MAX_ITER
        error('error in CSFB_cluster_Louvain: The number of maximum iterations has been attained. This dirty hack does not work in your case. See more comments in the README file');
    end
    disconnected_subgraph = false;
    COM=uint32(CSFB_run_Louvain(A));
    for i=1:max(COM)
        nodes_in_com = find(COM == i);
        if ~is_connected(A(nodes_in_com,nodes_in_com))
            disconnected_subgraph = true;
            delete('temp/com_level*');
            delete('temp/graph.tree');
            break;
            %warning('warning: disconnected subgraph!');
        end
    end
end

rmdir('temp','s');
