% The following code is the analysis cascade of a signal decomposition f 
% defined on a graph whose adjacency matrix is A. 
% 
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function [f0,Ai,COMi,BFf]=CSFB_compute_all_levels(f,A,clustering_method,depth)

%% some warnings
if depth<2
    error('depth must be superior to 1')
end
if strcmp(clustering_method,'regular_2d_grid')
    s=size(A); len=length(A)/(4^(depth-1));
        if ((s(1)~=s(2)) || (round(len)~=len))
            error('You cannot use ''regular_2d_grid'': the matrix is not square or its length is not dividable by 4^(depth-1)');
        end
end

%% choice clustering method

if strcmp(clustering_method,'Louvain')
    cluster_method=@(x,y) CSFB_cluster_Louvain(x);
elseif strcmp(clustering_method,'Ed_Aw_Louvain')   
    cluster_method=@(x,y) CSFB_cluster_f_Louvain(x,y); 
elseif strcmp(clustering_method,'regular_2d_grid')  
    cluster_method=@(x,y) CSFB_cluster_regular_2d_grid(x);     
elseif strcmp(clustering_method,'Infomap')
    cluster_method=@(x,y) CSFB_cluster_infomap(x);
elseif strcmp(clustering_method,'Ed_Aw_Infomap')
    cluster_method=@(x,y) CSFB_cluster_f_infomap(x,y);    
else
    error('error in compute_all_levels : your clustering_method is not supported');
end

%% initialisation

A0={}; f0={}; COMi={}; Ai={};
A0{1}=A; f0{1}=f; l=2;
BFf(1)=length(f);

%% compute all levels

% loop until the desired depth is achieved or until the approximation
% signal is of size 1: 
while (length(f0{l-1})~=1)&&(l-1<depth)
    % get partition
    COMi{l}=cluster_method(A0{l-1},f0{l-1});
    % and order its labels:
    auxCOM=COMi{l};K=1;
    for i=1:length(auxCOM)
        aux=auxCOM(i);
        if aux>K
            fi=(auxCOM==K);
            auxCOM(auxCOM==aux)=K;
            auxCOM(fi)=aux;
            K=K+1;
        elseif aux==K
            K=K+1;
        end
    end
    COMi{l}=auxCOM;
    % compute signal and structure and next level:
    [f0{l},Ai{l-1},A0{l},fHF{l}]=CSFB_compute_next_level(f0{l-1},A0{l-1},COMi{l});
    % number of low-pass coefficients at  level l:
    BFf(l)=length(f0{l});
    l=l+1;
end

%% organize signal output:

faux=f0{1};
K=0;
for i=2:l-1
    faux(1:length(f0{i}),i)=f0{i};
    faux(end-K-length(fHF{i})+1:end-K,i:l-1)=repmat(fHF{i},1,l-i);
    K=K+length(fHF{i});
end
f0=num2cell(faux,1);

