% The following code is an analysis block of a level of the analysis
% cascade
% 
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function [f0,Ai,A0,f1]=CSFB_compute_next_level(f,A,COM)

% size of each subgraph:
[Nk,~] = histc(COM,unique(COM));

% number of channels:
Ntilde=max(Nk);

% separate Adjacency matrix in Aint and Aext
[Ai,Ae]=CSFB_separate_Adjacency(A,COM);

% get local operators
[S,Q]=CSFB_get_Q_ordered(Ai,COM,Nk,Ntilde,'next');

% first channel approximation of f:
f0=Q{1}'*f;

% other channel details of f:
f1=cell(1,Ntilde);
for i=2:Ntilde
    f1{i}=Q{i}'*f;
end
f1=cell2mat(f1');

% next level structure:
A0 = S{1}'*Ae*S{1};

% force symmetrisation: (due to numerical precision, S{1}'*Ae*S{1} is not always exactly symmetrical)
A0 = (A0 + A0') ./ 2;

end
