% The following code computes local operators given a subgraph
% decompisition COM and a intra-subgraph adjacency matrix Ai.
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function [S,Q]=CSFB_get_Q_ordered(Ai,COM,Nk,Ntilde,direction)

global norm_Q_CSFB;

if norm_Q_CSFB==2
    
    K=max(COM);
    N=length(COM);
    
    MQ=zeros(Ntilde,1);
    xQ=cell(1,Ntilde); yQ=cell(1,Ntilde); valQ=cell(1,Ntilde);
    xS=cell(1,Ntilde); yS=cell(1,Ntilde); valS=cell(1,Ntilde);
    Ncur=N;
    for i=1:Ntilde
        Ncur=Ncur-(K-sum(Nk>i-1))*(i-1);
        xQ{i}=zeros(Ncur,1); yQ{i}=zeros(Ncur,1); valQ{i}=zeros(Ncur,1);
    end
    
    LL=zeros(Ntilde,1);
    if strcmp(direction,'next')
        for k=1:K
            mems=(COM==k);
            memslab=find(mems);
            Nmemslab=length(memslab);
            Ak=full(Ai(mems,mems));
            Lk=CSFB_create_laplacian(Ak,'combinatorial');
            %             Lk=full(Li(mems,mems));
            %             [Qaux,lambdaaux] = eig(Lk);
            [Qaux,lambdaaux,~] = svd(Lk);
            lambdaaux=diag(lambdaaux);
            [~,ind] = sort(lambdaaux);
            Qaux=Qaux(:,ind) ;
            %  lambdaaux(1) = 0 ; % this is always the case whatever the chosen Laplacian
            %  lambdaaux(lambdaaux<0) = 0 ; % all three Laplacians are positive semi-definite
            if sign(Qaux(1,1))==-1
                Qaux=-Qaux;
            end
            for i=1:Nk(k)
                xQ{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yQ{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i);
                %valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i)./sqrt(sum(Qaux(:,i)~=0));
                %xS{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(Nmemslab,1);
                %valS{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i).*sqrt(sum(Qaux(:,i)~=0));
            end
            MQ(1:Nk(k))=MQ(1:Nk(k))+1;
            LL(1:Nk(k))=LL(1:Nk(k))+Nmemslab;
        end
        
    elseif strcmp(direction,'prev')
        for k=1:K
            mems=(COM==k);
            memslab=find(mems);
            Nmemslab=length(memslab);
            Ak=full(Ai(mems,mems));
            Lk=CSFB_create_laplacian(Ak,'combinatorial');
            %             Lk=full(Li(mems,mems));
            %             [Qaux,lambdaaux] = eig(Lk);
            [Qaux,lambdaaux,~] = svd(Lk);
            lambdaaux=diag(lambdaaux);
            [~,ind] = sort(lambdaaux);
            Qaux=Qaux(:,ind) ;
            %  lambdaaux(1) = 0 ; % this is always the case whatever the chosen Laplacian
            %  lambdaaux(lambdaaux<0) = 0 ; % all three Laplacians are positive semi-definite
            if sign(Qaux(1,1))==-1
                Qaux=-Qaux;
            end
            for i=1:Nk(k)
                xQ{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yQ{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i);
                %valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i)./sqrt(sum(Qaux(:,i)~=0));
                %xS{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(Nmemslab,1);
                %valS{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i).*sqrt(sum(Qaux(:,i)~=0));
            end
            MQ(1:Nk(k))=MQ(1:Nk(k))+1;
            LL(1:Nk(k))=LL(1:Nk(k))+Nmemslab;
        end
    end
    
    
    Q=cell(1,Ntilde);
    S=cell(1,Ntilde);
    for i=1:Ntilde
        Q{i}=sparse(xQ{i},yQ{i},valQ{i},N,max(yQ{i}));
        S{i}=sparse(xQ{i},yQ{i},valS{i},N,max(yQ{i}));
        %S{i}=sparse(xS{i},yS{i},valS{i},N,max(yQ{i}));
    end

elseif norm_Q_CSFB==1
    
     
    K=max(COM);
    N=length(COM);
    
    MQ=zeros(Ntilde,1);
    xQ=cell(1,Ntilde); yQ=cell(1,Ntilde); valQ=cell(1,Ntilde);
%     xS=cell(1,Ntilde); yS=cell(1,Ntilde); 
    valS=cell(1,Ntilde);
    Ncur=N;
    for i=1:Ntilde
        Ncur=Ncur-(K-sum(Nk>i-1))*(i-1);
        xQ{i}=zeros(Ncur,1); yQ{i}=zeros(Ncur,1); valQ{i}=zeros(Ncur,1);
    end
    
    LL=zeros(Ntilde,1);
    if strcmp(direction,'next')
        for k=1:K
            mems=(COM==k);
            memslab=find(mems);
            Nmemslab=length(memslab);
            Ak=full(Ai(mems,mems));
            Lk=CSFB_create_laplacian(Ak,'combinatorial');
            %             Lk=full(Li(mems,mems));
            %             [Qaux,lambdaaux] = eig(Lk);
            [Qaux,lambdaaux,~] = svd(Lk);
            lambdaaux=diag(lambdaaux);
            [lambdaaux,ind] = sort(lambdaaux);
            Qaux=Qaux(:,ind) ;
            if sign(Qaux(1,1))==-1
                Qaux=-Qaux;
            end
%             [~,Qaux]=CSFB_condition_if_multiplicity(Qaux,lambdaaux,1e-14);
            Qaux=Qaux./repmat(sum(abs(Qaux)),Nmemslab,1);
             
            for i=1:Nk(k)
                xQ{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yQ{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                %                 valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qauxinv(:,i);
                valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i);%sum(Qaux(:,i)~=0));
                %xS{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(Nmemslab,1);
                %valS{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i).*sqrt(sum(Qaux(:,i)~=0));
            end
            MQ(1:Nk(k))=MQ(1:Nk(k))+1;
            LL(1:Nk(k))=LL(1:Nk(k))+Nmemslab;
        end
        
    elseif strcmp(direction,'prev')
        for k=1:K
            mems=(COM==k);
            memslab=find(mems);
            Nmemslab=length(memslab);
            Ak=full(Ai(mems,mems));
            Lk=CSFB_create_laplacian(Ak,'combinatorial');
            %             Lk=full(Li(mems,mems));
            %             [Qaux,lambdaaux] = eig(Lk);
            [Qaux,lambdaaux,~] = svd(Lk);
            lambdaaux=diag(lambdaaux);
            [lambdaaux,ind] = sort(lambdaaux);
            Qaux=Qaux(:,ind) ;
            if sign(Qaux(1,1))==-1
                Qaux=-Qaux;
            end
%             [~,Qaux]=CSFB_condition_if_multiplicity(Qaux,lambdaaux,1e-14);
            Qaux=Qaux./repmat(sum(abs(Qaux)),Nmemslab,1);
            Qauxinv=inv(Qaux);
            Qauxinv=Qauxinv';
            for i=1:Nk(k)
                xQ{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yQ{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                %                 valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i);
                valQ{i}(LL(i)+1:LL(i)+Nmemslab)=Qauxinv(:,i);%sum(Qaux(:,i)~=0));
                %xS{i}(LL(i)+1:LL(i)+Nmemslab)=memslab; yS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(size(memslab)).*(MQ(i)+1);
                valS{i}(LL(i)+1:LL(i)+Nmemslab)=ones(Nmemslab,1);
                %valS{i}(LL(i)+1:LL(i)+Nmemslab)=Qaux(:,i).*sqrt(sum(Qaux(:,i)~=0));
            end
            MQ(1:Nk(k))=MQ(1:Nk(k))+1;
            LL(1:Nk(k))=LL(1:Nk(k))+Nmemslab;
        end
    end
    
    
    Q=cell(1,Ntilde);
    S=cell(1,Ntilde);
    for i=1:Ntilde
        Q{i}=sparse(xQ{i},yQ{i},valQ{i},N,max(yQ{i}));
        S{i}=sparse(xQ{i},yQ{i},valS{i},N,max(yQ{i}));
        %S{i}=sparse(xS{i},yS{i},valS{i},N,max(yQ{i}));
    end
    
    
else
    error('CSFB_get_Q_ordered: norm_Q_CSFB should be equal to 0 or 1');
end