% The following code runs Infomap on adj
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function [COM] = CSFB_infomap(adj,self,varargin)

global GLOBAL_CSFB; global strategy_CSFB;

if nargin==2
    MS_flag=0;
elseif nargin==3
    MS_flag=varargin{1};
else error('error in infomap.m : function allows 2 or 3 inputs');
end

mkdir temp

%% Set the path and command line name
dir_path =[GLOBAL_CSFB,filesep,'3rd_party',filesep,'Infomap',filesep];
command = 'Infomap';

command = [dir_path,command];
if MS_flag==0
    if self==1
        command = [command,' temp/adj.txt temp/ -N 2 --clu --include-self-links'];
    elseif self==0
        command = [command,' temp/adj.txt temp/ -N 2 --clu'];
    else error('infomap.m: self flag must be equal to 0 or to 1');
    end
else
    if self==1
        command = [command,' temp/adj.txt temp/ -N 2 --include-self-links'];
    elseif self==0
        command = [command,' temp/adj.txt temp/ -N 2'];
    else error('infomap.m: self flag must be equal to 0 or to 1');
    end
    %else error('infomap.m: MS flag must be equal to 0 or to 1');
end

%% create file list in text file:
[I,J,val]=find(adj);
I(end+1)=length(adj)-1; J(end+1)=length(adj); val(end+1)=0;
I(end+1)=length(adj); J(end+1)=length(adj)-1; val(end+1)=0;
Alist=[I,J,val];
dlmwrite('temp/adj.txt',Alist,'delimiter', ' ','precision',1+ceil(log10(length(adj))));

%% Call community detection algorithm
[~,~] = system(command);


%% Load data and create data structure
if MS_flag==0
    fid = fopen('temp/adj.clu','rt');
    if fid==-1
        com=ones(length(adj),1);
        delete('temp/adj.net');
        delete('temp/adj.tree')
        rmdir('temp');
        return;
    else
        com = textscan(fid,'%f','Headerlines',2);
        fclose(fid);
        com = com{1};
        % Delete files
        delete('temp/adj.net')
        delete('temp/adj.clu')
        %delete('temp/adj.tree')
        rmdir('temp','s')
    end
elseif MS_flag==1
    fid = fopen('temp/adj.tree','r');
    if fid==-1
        com=ones(length(adj),1);
        rmdir('temp','s');
        return;
    else
        fclose(fid);
        file=fileread('temp/adj.tree');
        expr='((\d+:)+\d+)';
        aux = regexp(file,expr,'match');
        expr='\s\d+\n';
        aux2 = str2num(cell2mat(regexp(file,expr,'match')));
        aux2=aux2+1;
        aux=cellfun(@CSFB_minus_two,aux,'uniformoutput',false);
        %list_cluster=unique(aux);
        %com=zeros(size(aux'));
        [com, ~] = grp2idx(aux);
        %    for i=1:length(list_cluster)
        %        %com(cellfun(@(x)~isempty(x),strfind(aux, list_cluster{i})))=i;
        %        %com=com+i.*ismember(aux,list_cluster{i});
        %        com=com+i.*(gi==i);
        %    end
        [~,ind]=sort(aux2);
        com=com(ind);
        rmdir('temp','s');
    end
elseif MS_flag==2
    fid = fopen('temp/adj.tree','r');
    if fid==-1
        COM=ones(length(adj),1);
        warning('CSFB_infomap: For some reason, Infomap did not output a result. Results may be erroneous.');
        rmdir('temp','s');
        return;
    else
        fclose(fid);
        file=fileread('temp/adj.tree');
        %expr='((\d+:)+\d+)'; aux = regexp(file,expr,'match');
        expr='\n[^#\s]+'; aux = regexp(file,expr,'match');
        expr='\s\d+\n'; auxnode = str2num(cell2mat(regexp(file,expr,'match')));
        %expr='\d+\n'; aux2=cellfun(@str2num,regexp(file,expr,'match'));
        %aux2=aux2+1;
        
        num_levels=cellfun(@(x)sum(x==':'),aux);
        for l=max(num_levels)-1:-1:min(num_levels)
            nod=find(num_levels<=l);
            auxnod=cellfun(@(x)[x ':1'],aux(nod),'uniformoutput', false);
            aux(nod)=auxnod;
        end
        
        
        level_max=max(num_levels);
        
        COM=zeros(length(adj),level_max);
        sizeCOM=cell(1,level_max);
        auxmat=cell2mat(aux);
        for level=1:level_max
            expr=['\n((\d+:){' num2str(level-1) '})\d+'];
            aux2=regexp(auxmat,expr,'match');
            %aux2 = cellfun(@cell2mat,regexp(aux,expr,'match'),'uniformoutput',false);
            [COM(:,level), ~] = grp2idx(aux2);
            [sizeCOM{level},~] = histc(COM(:,level),unique(COM(:,level)));
%             sizeCOM{level}=hist(COM(:,level),max(COM(:,level)))';
        end
        
        % keep the highest scale level whose largest community has less than 1000
        % nodes:
        if level_max>0
            if strcmp(strategy_CSFB,'largest')
                f=find(cellfun(@max,sizeCOM)<1000,1,'first');%<1000
            elseif strcmp(strategy_CSFB,'smallest')
                f=find(cellfun(@min,sizeCOM)~=1,1,'last');%<1000
            else error('CSFB_infomap: strategy_CSFB should be either ''largest'' or ''smallest''');
            end
            COM=COM(:,f);
        else
            COM=ones(N,1);
        end
        
        [~,ind]=sort(auxnode);
        COM=COM(ind)';
        rmdir('temp','s');
        
    end
end