% the following code is copied from bits and pieces of Demos part of the 3rd
% party Graph Bior Filterbanks implementaton (see README.txt)

function [Data,f_hat1,BFf] = NLA_GrBior(filename,max_level,nnz_factor,edgemap,vanishing_moments,norm_type)

theta = 2; % number of bipartite graphs
Fmax = 2^theta; % Graph Coloring
N = zeros(max_level,1); %size of bipartite graphs at each level
% edgemap = 0; % uses edge-map if 1, use regular 8-connected graph otherwise
% norm_type = 'asym'; % use 'sym'for symmetric Normalized Laplacian matrix
% S = sprintf('%s%d%s%d%s%d%s%f%s%d', '# decomposition levels = ', max_level,'.  Biorthtype = ', vanishing_moments(1),' / ',vanishing_moments(2), '.   nnz_factor = ',nnz_factor,'.   Using edgemap = ', edgemap);
% disp(S);

%% Section 1: Image Graph Formulation
% Graph Signal
tempI = imfinfo(filename);
filetype = tempI.Format;
Data = imread(filename,filetype);
if length(size(Data)) == 3
    Data = rgb2gray(Data);
end
Data = double(Data);%/255;
[m n] = size(Data);
m = min([m,n]);
s_im = floor(m/2^max_level)*2^max_level;
% s_im = 10;
Data = Data(1:s_im,1:s_im);
f = Data(:);
% f = f/norm(f);

% Graphs
[bptG Colorednodes, beta_dist loc] = image_graphs_multi(Data,max_level,edgemap);  % image graphs


%% Section 2: Filterbank implementation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Normalized Laplacian Matrices for Each Bpt graph
% disp('Computing normalized Laplacian matrices for each subgraph...');
Ln_bpt = cell(max_level,theta);
switch norm_type
    case 'sym'
        for level = 1:max_level
            N(level) = length(bptG{level,1});
            for i = 1:theta
                d1 = sum(bptG{level,i},2);
                d1(d1 == 0) = 1; % for isolated nodes
                d1_inv = d1.^(-0.5);
                D1_inv = spdiags(d1_inv, 0, N(level), N(level));
                An = D1_inv*bptG{level,i}*D1_inv;
                An = 0.5*(An + An');
                Ln_bpt{level,i} = speye(N(level)) - An;
            end
        end
    case 'asym'
        for level = 1:max_level
            N(level) = length(bptG{level,1});
            for i = 1:theta
                d1 = sum(bptG{level,i},2);
                d1(d1 == 0) = 1; % for isolated nodes
                d1_inv = d1.^(-1);
                D1_inv = spdiags(d1_inv, 0, N(level), N(level));
                An = D1_inv*(0.5*(bptG{level,i} + bptG{level,i}'));
                Ln_bpt{level,i} = speye(N(level)) - An;
            end
        end
    otherwise
        disp('Unknown normalization option')
        return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% design biorthogonal kernels

Nd = vanishing_moments(1);
Nr = vanishing_moments(2);
% S = sprintf('%s%d%s ' , ' Computing a ',filterlen,' ^th order approximation of Meyer kernel');
% disp(S)
% % % % % % % % % % % [hi_d,lo_d] = biorth_kernel(Nd,Nr);
[lo_d,hi_d] = biorth_kernel_new(Nd,Nr); % Nd zeros for lowpass Nr zeros for highpass
filterlen_hi = length(roots(hi_d));
filterlen_lo = length(roots(lo_d));
h0 = @(x)(polyval(lo_d,x));
h1 = @(x)(polyval(hi_d,x));
g0 = @(x)(polyval(hi_d,2 - x));
g1 = @(x)(polyval(lo_d,2 - x));
arange = [0 2];
c_d{1}=sgwt_cheby_coeff(h0,filterlen_lo,filterlen_lo+1,arange);
c_d{2}=sgwt_cheby_coeff(h1,filterlen_hi,filterlen_hi+1,arange);
c_r{1}=sgwt_cheby_coeff(g0,filterlen_hi,filterlen_hi+1,arange);
c_r{2}=sgwt_cheby_coeff(g1,filterlen_lo,filterlen_lo+1,arange);



dfactor = cell(max_level,1);
for level = 1:max_level
    for i = 1:theta
        d1 = sum(bptG{level,i},2);
        d1(d1 == 0) = 1; % for isolated nodes
        d1_sqrt = d1.^(0.5);
        dfactor{level}(:,i) = full(d1_sqrt);
    end
end
GC = compute_GC_Bior(Ln_bpt,dfactor, hi_d,lo_d, norm_type);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Filterbank Output at each channel
% disp('Computing wavelet transform coefficients ...')
f_w = cell(max_level,1);
function_norm = ones(max_level,1);
Channel_Name = cell(max_level,Fmax);
for level = 1:max_level
    f_w{level} = zeros(N(level),Fmax);
    for i = 1:Fmax
        if level == 1
            function_norm(level) = norm(f);
            tempf_w = f/function_norm(level);
        else
            function_norm(level) = norm(f_w{level-1}(Colorednodes{level-1,1},1));
            tempf_w = f_w{level-1}(Colorednodes{level-1,1},1)/function_norm(level);
        end
        for j = 1: theta
            if beta_dist{level}(i,j) == 1
                tempf_w = sgwt_cheby_op(tempf_w,Ln_bpt{level,j},c_d{1},arange);
                tempf_w = tempf_w./GC{level,j}(:,1);
                Channel_Name{level,i} = strcat(Channel_Name{level,i},'L');
            else
                tempf_w = sgwt_cheby_op(tempf_w, Ln_bpt{level,j},c_d{2},arange);
                tempf_w = tempf_w./GC{level,j}(:,2);
                Channel_Name{level,i} = strcat(Channel_Name{level,i},'H');
            end
        end
        f_w{level}(Colorednodes{level,i},i) = tempf_w(Colorednodes{level,i});
    end
end

%% Section 3: Non-linear Approximation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f_w = find_kbest(f_w, nnz_factor, max_level, Colorednodes); % threshold for nCoeffs best wavelet coefficients

for i=1:max_level-1
    f_w{i}(:,1)=zeros(size(f_w{i}(:,1)));
end
    
%% Section 4: Reconstruction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reconstruct Signals in Original Graph Domain by using Coefficients
% Compute Filterbank Output at each channel
% disp('Reconstructed signals using single channel coefficients ...')

f_hat1 = zeros(N(1),1);
for d_level = max_level
    f_hat = f_w;
    for level = d_level: -1 :1
        % Channel_Name = cell(Fmax,1);
        for i = 1:Fmax
            tempf_hat = f_hat{level}(:,i);
            for j = theta: -1: 1
                if beta_dist{level}(i,j) == 1
                    tempf_hat = GC{level,j}(:,1).*tempf_hat;
                    tempf_hat = sgwt_cheby_op(tempf_hat,Ln_bpt{level,j},c_r{1},arange);
                else
                    tempf_hat = GC{level,j}(:,2).*tempf_hat;
                    tempf_hat = sgwt_cheby_op(tempf_hat,Ln_bpt{level,j},c_r{2},arange);
                end
            end
            f_hat{level}(:,i) = tempf_hat;
        end
        f_hat{level} = sum(f_hat{level},2)*function_norm(level);
        %         f_hat{level} = f_hat{level}(:,1);
        
        if level >1
            f_hat{level-1}(Colorednodes{level-1,1},1) = f_hat{level};
        end
    end
    f_hat1= f_hat{level};
end

BFf(1)=N(1);
BFf(2)=length(Colorednodes{max_level}(:,1));

