% the following code is copied from bits and pieces of Demos part of the 3rd
% party Graph Bior Filterbanks implementaton (see README.txt)

function [f_hat1] = denoise_graph_bior(A,f,thresh,norm_type,vanishing_moments)


% filterlen = 12;
% % Set parameters
% max_level = 1; % number of decomposition levels
% filterlen = opt.filterLen; % filter length
% norm_type = 'asym';
%GC_module = opt.GC;
% nnz_factor = 0.03; % fraction of non-zero highpass coefficients

% theta = 2; % number of bipartite graphs
% Fmax = 2^theta; % Graph Coloring
% N = zeros(max_level,1); %size of bipartite graphs at each level

%% Section 1: Graph Formulation

% % Minnesota traffic graph
% [A xy] = Minnesota_traffic_graph(); % Minnesota Traffic Graph

%% Section 2: Bipartite subgraph decomposition

% Graph Coloring: F is the output coloring vector
%%%%%%%%%%%%%
F = BSC(A); % Backtracking Sequential Coloring Algorithm: Exact but slow
% F = LF(A);   % Greedy Largest Degree First Coloring: Fast but inaccurate
% F = DSATUR(A); % Greedy Coloring based on improved heuristic: Moderate speed and accuracy
% load min_coloring %  Preloaded Coloring for Minnesota graph
% %%%%%%%%%%%%%%
% 
% load min_graph_asym
% load min_graph_CN
N = length(A);
[beta bptG beta_dist Colorednodes]= harary_decomp(A,F);
theta = size(beta,2);
Fmax = size(beta_dist,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% design biorthogonal kernels

Nd = vanishing_moments(1);
Nr = vanishing_moments(2);
% S = sprintf('%s%d%s ' , ' Computing a ',filterlen,' ^th order approximation of Meyer kernel');
% disp(S)
[lo_d,hi_d] = biorth_kernel_new(Nd,Nr); % Nd zeros for lowpass Nr zeros for highpass
filterlen_hi = length(roots(hi_d));
filterlen_lo = length(roots(lo_d));
g0 = @(x)(polyval(lo_d,x));
g1 = @(x)(polyval(hi_d,x));
h0 = @(x)(polyval(hi_d,2 - x));
h1 = @(x)(polyval(lo_d,2 - x));

arange = [0 2];
c_d{1}=sgwt_cheby_coeff(h0,filterlen_lo,filterlen_lo+1,arange);
c_d{2}=sgwt_cheby_coeff(h1,filterlen_hi,filterlen_hi+1,arange);
c_r{1}=sgwt_cheby_coeff(g0,filterlen_hi,filterlen_hi+1,arange);
c_r{2}=sgwt_cheby_coeff(g1,filterlen_lo,filterlen_lo+1,arange);

%% Section 3: Graph Signal
% load min_graph_signal % Preloaded graph signal for Minnesota traffic
% load min_graph_ne_CS
% 
% f_orig = f;
% 
% if ~exist('f_noisy', 'var')
%     f = f  + sigma * randn(size(f));
% else
%     f = f_noisy;
% end
Ln_bpt = cell(1,theta);
switch norm_type
    case 'sym'
        for i = 1:theta
            %     bptG(:,:,i) = 0.5*(bptG(:,:,i) + bptG(:,:,i)');
            d1 = sum(bptG(:,:,i),2);
            d1(d1 == 0) = 1; % for isolated nodes
            d1_inv = d1.^(-0.5);
            D1_inv = diag(d1_inv);
            An = D1_inv*bptG(:,:,i)*D1_inv;
            An = 0.5*(An + An');
            %     d1_inv = d1.^(-1);
            %     D1_inv = diag(d1_inv);
            %     An = D1_inv*bptG(:,:,i);
            Ln_bpt{1,i} = eye(N) - An;
        end
    case 'asym'
        for i = 1:theta
            bptG(:,:,i) = 0.5*(bptG(:,:,i) + bptG(:,:,i)');
            d1 = sum(bptG(:,:,i),2);
            d1(d1 == 0) = 1; % for isolated nodes
            d1_inv = d1.^(-1);
            D1_inv = diag(d1_inv);
            An = D1_inv*bptG(:,:,i);
            Ln_bpt{1,i} = eye(N) - An;
        end
    otherwise
        disp('Unknown normalization option')
        return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Filterbank Output at each channel
% disp('Computing wavelet transform coefficients ...')

nchannel = 2;
nsubband = nchannel^2; % # of subbands

f_w = cell(1, nsubband);

% function_norm = norm(f);
tempf_w = f;%/function_norm;

for i = 0: nchannel-1
    
    tempf_w1 = sgwt_cheby_op(tempf_w, Ln_bpt{1}, c_d{i+1}, arange);
    
    tempf_w2 = cell(1, nchannel);
    
    for j = 1: nchannel
        
        if ~isempty(Colorednodes{i*nchannel + j})
            tempf_w2{j} = sgwt_cheby_op(tempf_w1,Ln_bpt{2}, c_d{j}, arange);
            
            f_w{i*nchannel + j} = zeros(size(tempf_w));
            f_w{i*nchannel + j}(Colorednodes{i*nchannel + j}) = tempf_w2{j}(Colorednodes{i*nchannel + j});
            
        end
        
    end
end


% thresh = 3 * sigm;

for s = 2: length(f_w)
    if ~isempty(f_w{s})
        f_w{s} = f_w{s} .* (abs(f_w{s}) > thresh);% * E{s});
    end
end

%
f_w_approx = f_w;
% for k = 2:16 %[3, 4, 7:16] %
%     f_w_approx{k} = zeros(size(f_w{k}));
% end

%% Section 4: Reconstruction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reconstruct Signals in Original Graph Domain by using Coefficients
% Compute Filterbank Output at each channel
% disp('Reconstructed signals using single channel coefficients ...')
%
% f_w_approx = f_w;

f_hat = cell(1, nchannel);
f_hat1 = zeros(size(f));

for i = 0: nchannel-1
    
    f_hat{i+1} = zeros(size(f));
    
    for j = 1: nchannel
        
        if ~isempty(f_w_approx{i*nchannel + j})
            f_hat{i+1} = f_hat{i+1} + sgwt_cheby_op(f_w_approx{i*nchannel + j},Ln_bpt{2}, c_r{j}, arange);
        end
        
    end
    
    f_hat1 = f_hat1 + sgwt_cheby_op(f_hat{i+1}, Ln_bpt{1}, c_r{i+1}, arange);
    
end

% BFf(1)=length(f_hat1);
% BFf(2)=sum(f_w{1}(:,1)~=0);

% f_hat1 = f_hat1;% * function_norm;
% 
% MSE= mean((f_orig - f_hat1).^2);
% MSE_noisy = mean((f_orig - f).^2);
% P_rms = mean(f_orig.*f_orig);
% SNR = 10*log10(P_rms/MSE);
% SNR_noisy = 10*log10(P_rms/MSE_noisy);
% S = sprintf('%s%.3f','MSE of noisy graph signal = ' ,MSE_noisy);
% disp(S);
% S = sprintf('%s%.2f%s','SNR of noisy signal = ' ,SNR_noisy, ' dB');
% disp(S);
% S = sprintf('%s%.3f','MSE of denoised signal by GraphBior = ' ,MSE);
% disp(S);
% S = sprintf('%s%.2f%s','SNR of denoised signal by GraphBior = ' ,SNR, ' dB');
% disp(S);
% 
% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % Scatter Plots
% % use same scale for high-pass coefficients
% 
% f_lowpass = f_w{1};
% f_lowpass = f_lowpass(Colorednodes{1});
% Data = [f_orig(:);f_lowpass];
% clim = [min(Data) max(Data)];
% f_orig = (f_orig - clim(1))/(clim(2)-clim(1));
% f_orig = 2*f_orig -1;
% f = (f - clim(1))/(clim(2)-clim(1));
% f = 2*f -1;
% f_hat1 = (f_hat1 - clim(1))/(clim(2)-clim(1));
% f_hat1 = 2*f_hat1 -1;
% 
% f_lowpass = (f_lowpass - clim(1))/(clim(2)-clim(1));
% f_lowpass = 2*f_lowpass -1;
% figure;
% show_wavelet(f_orig,xy(:,1),xy(:,2));
% title('Input Signal')
% axis auto
% 
% figure;
% show_wavelet(f,xy(:,1),xy(:,2));
% title('Noisy Signal')
% axis auto
% 
% figure;
% show_wavelet(f_hat1,xy(:,1),xy(:,2));
% title('Denoised Signal by Critically-sampled Biorthogonal GFB')
% axis auto
% 
% % figure;
% % show_wavelet(f_hat1,xy(:,1),xy(:,2));
% % S = sprintf('Wavelet coeffs of LL channel');
% % title(S)
% % axis auto
% %
% % % wavelet coeffs
% % figure;
% % show_wavelet(f_lowpass,xy(Colorednodes{1},1),xy(Colorednodes{1},2));
% % S = sprintf('Wavelet coeffs of LL channel');
% % title(S)
% % axis auto
% %
% %
% % f_highpass = f_w;
% % w_d = [];
% % for i = 0: nchannel-1
% %     for j = 1: length(h)
% %         ind_sub = nchannel/2 * floor(i/2) + floor((j+1)/2);
% %
% %         if i == 0 && j ==1
% %             f_highpass{i*nchannel + j} = [];
% %         else
% %             if ~isempty(Colorednodes{ind_sub})
% %                 f_highpass{i*nchannel + j} = f_highpass{i*nchannel + j}(Colorednodes{ind_sub});
% %                 w_d = [w_d; f_highpass{i*nchannel + j}];
% %             end
% %         end
% %
% %     end
% % end
% %
% % for k = 1: nsubband
% %     if ~isempty(f_highpass{k})
% %         f_highpass{k} = abs(f_highpass{k})/norm(w_d);
% %     end
% % end
% %
% % position_ind = [0 2 5 6 3 4 7 8 9 10 13 14 11 12 15 16];
% %
% % figure;
% % for i = 0: nchannel-1
% %     for j = 1: length(h)
% %         ind_sub = nchannel/2 * floor(i/2) + floor((j+1)/2);
% %
% %         if i == 0 && j ==1
% %         else
% %             subplot(4, 4, position_ind(i*nchannel + j));
% %             show_wavelet(f_highpass{i*nchannel + j}, xy(Colorednodes{ind_sub},1), xy(Colorednodes{ind_sub},2))
% %             S = sprintf('%s%d', 'Channel #', i*nchannel + j - 1);
% %             title(S)
% %             axis auto
% %         end
% %
% %     end
% % end
% 
% 
