function [G]=gsp_update_laplacian(G,lap_type)
%GSP_UPDATE_LAPLACIAN  Update a graph's Laplacian
%   Usage:  G=gsp_update_laplacian(G,lap_type);
%
%   Input parameters:
%         G          : Graph structure.
%         lap_type   : Options include 'combinatorial' or 'normalized'.
%   Output parameters:
%         G          : Updated graph structure.
%
%   'gsp_update_laplacian(G,lap_type)' updates or initializes a graph
%   Laplacian. The input graph structure must contain a weighted adjacency
%   matrix. The function also removes any eigenvectors, eigenvalues, and
%   maximum eigenvalue that had been computed for a previous Laplacian.
%
%   See also:  
%
%   Demos:  

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
if ~isfield(G,'W')
    error('Weighted adjacency matrix not specified');
end

% Create graph Laplacian
switch lap_type
    case 'combinatorial'
        G.L=G.D-G.W;
        G.lap_type='combinatorial';
    case 'normalized'
        G.L=G.D-G.W;
        factor=diag(diag(G.D).^(-.5));
        G.L=factor*G.L*factor;
        G.lap_type='normalized';
    otherwise
        error('Unknown Laplacian type')
end

if isfield(G,'lmax')
    rmfield(G,'lmax')
end

if isfield(G,'U')
    rmfield(G,'U')
end

if isfield(G,'E')
    rmfield(G,'E')
end